# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: morai_sim_info.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
import morai_type_pb2 as morai__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14morai_sim_info.proto\x12\rmorai_sim_api\x1a\x1bgoogle/protobuf/empty.proto\x1a\x10morai_type.proto\"s\n\x11\x43\x61tegoryObstacles\x12\x0f\n\x07vehicle\x18\x01 \x01(\x08\x12\x12\n\npedestrian\x18\x02 \x01(\x08\x12\x10\n\x08obstacle\x18\x03 \x01(\x08\x12\x13\n\x0bspawn_point\x18\x04 \x01(\x08\x12\x12\n\nmap_object\x18\x05 \x01(\x08\"q\n\x0f\x41vailableObject\x12\x0f\n\x07vehicle\x18\x01 \x03(\t\x12\x12\n\npedestrian\x18\x02 \x03(\t\x12\x10\n\x08obstacle\x18\x03 \x03(\t\x12\x13\n\x0bspawn_point\x18\x04 \x03(\t\x12\x12\n\nmap_object\x18\x05 \x03(\t2\xce\x03\n\x0cMoraiSimInfo\x12K\n\x13GetSimulatorVersion\x12\x16.google.protobuf.Empty\x1a\x1a.morai_sim_api.StringValue\"\x00\x12\x43\n\x0cGetTimestamp\x12\x16.google.protobuf.Empty\x1a\x19.morai_sim_api.Int64Value\"\x00\x12G\n\x10GetAvailableMaps\x12\x16.google.protobuf.Empty\x1a\x19.morai_sim_api.StringList\"\x00\x12\x44\n\rGetActorTypes\x12\x16.google.protobuf.Empty\x1a\x19.morai_sim_api.StringList\"\x00\x12X\n\x12GetAvailableObject\x12 .morai_sim_api.CategoryObstacles\x1a\x1e.morai_sim_api.AvailableObject\"\x00\x12\x43\n\x0bGetDataPath\x12\x16.google.protobuf.Empty\x1a\x1a.morai_sim_api.StringValue\"\x00\x42&H\x01\xaa\x02!CoreLib.Grpc.Protobuf.MoraiSimAPIb\x06proto3')



_CATEGORYOBSTACLES = DESCRIPTOR.message_types_by_name['CategoryObstacles']
_AVAILABLEOBJECT = DESCRIPTOR.message_types_by_name['AvailableObject']
CategoryObstacles = _reflection.GeneratedProtocolMessageType('CategoryObstacles', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORYOBSTACLES,
  '__module__' : 'morai_sim_info_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.CategoryObstacles)
  })
_sym_db.RegisterMessage(CategoryObstacles)

AvailableObject = _reflection.GeneratedProtocolMessageType('AvailableObject', (_message.Message,), {
  'DESCRIPTOR' : _AVAILABLEOBJECT,
  '__module__' : 'morai_sim_info_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.AvailableObject)
  })
_sym_db.RegisterMessage(AvailableObject)

_MORAISIMINFO = DESCRIPTOR.services_by_name['MoraiSimInfo']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\001\252\002!CoreLib.Grpc.Protobuf.MoraiSimAPI'
  _CATEGORYOBSTACLES._serialized_start=86
  _CATEGORYOBSTACLES._serialized_end=201
  _AVAILABLEOBJECT._serialized_start=203
  _AVAILABLEOBJECT._serialized_end=316
  _MORAISIMINFO._serialized_start=319
  _MORAISIMINFO._serialized_end=781
# @@protoc_insertion_point(module_scope)



import numpy as np
from copy import deepcopy as dc
from spatialmath import base as smb

def get_tr(transform):
    tr = np.eye(4)
    tr[:3, :3] = get_r(transform.rotation)
    tr[:3, 3] = np.array([transform.location.x, transform.location.y, transform.location.z])
    return tr

def get_r(rotation):
    return smb.rpy2r(rotation.roll, rotation.pitch, rotation.yaw, unit="deg", order="zyx")

def r2rpy(R):
    return smb.tr2rpy(R, unit="deg", order="zyx")

def iso2unity(transform):
    u_transform = dc(transform)
    u_transform.location.x = -transform.location.y
    u_transform.location.y = transform.location.z
    u_transform.location.z = transform.location.x
    u_transform.rotation.roll = -transform.rotation.roll
    u_transform.rotation.pitch = transform.rotation.pitch
    u_transform.rotation.yaw = -transform.rotation.yaw

    return u_transform

def unity2iso(transform):
    i_transform = dc(transform)
    i_transform.location.x = transform.location.z
    i_transform.location.y = -transform.location.x
    i_transform.location.z = transform.location.y
    i_transform.rotation.roll = -transform.rotation.roll
    i_transform.rotation.pitch = transform.rotation.pitch
    i_transform.rotation.yaw = -transform.rotation.yaw

    return i_transform

def enu2world(transform):
    u_transform = dc(transform)
    u_transform.location.x = -transform.location.x
    u_transform.location.y = transform.location.z
    u_transform.location.z = -transform.location.y
    u_transform.rotation.roll = -transform.rotation.roll
    u_transform.rotation.pitch = transform.rotation.pitch
    u_transform.rotation.yaw = -transform.rotation.yaw + 270

    return u_transform

def world2enu(transform):
    i_transform = dc(transform)
    i_transform.location.x = -transform.location.x
    i_transform.location.y = -transform.location.z
    i_transform.location.z = transform.location.y
    i_transform.rotation.roll = -transform.rotation.roll
    i_transform.rotation.pitch = transform.rotation.pitch
    i_transform.rotation.yaw = -(transform.rotation.yaw-180)+90

    return i_transform



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: morai_sensor.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import morai_type_pb2 as morai__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12morai_sensor.proto\x12\rmorai_sim_api\x1a\x10morai_type.proto\"\xa6\x01\n\rSensorSetting\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12.\n\x0bsensor_type\x18\x02 \x01(\x0e\x32\x19.morai_sim_api.SensorType\x12(\n\x08position\x18\x03 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12(\n\x08rotation\x18\x04 \x01(\x0b\x32\x16.morai_sim_api.Vector3\"C\n\x11SensorSettingList\x12.\n\x08req_list\x18\x01 \x03(\x0b\x32\x1c.morai_sim_api.SensorSetting\"4\n\x0cRemoveSensor\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12\x11\n\tsensor_id\x18\x02 \x01(\t\"A\n\x10RemoveSensorList\x12-\n\x08req_list\x18\x01 \x03(\x0b\x32\x1b.morai_sim_api.RemoveSensor\"G\n\x0eSensorResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\ttarget_id\x18\x02 \x01(\t\x12\x11\n\tsensor_id\x18\x03 \x01(\t\"E\n\x12SensorResponseList\x12/\n\x08res_list\x18\x01 \x03(\x0b\x32\x1d.morai_sim_api.SensorResponse\"J\n\x11GTCylinderSetting\x12\x10\n\x08segments\x18\x01 \x01(\x05\x12\x14\n\x0cvertical_fov\x18\x02 \x01(\x02\x12\r\n\x05range\x18\x03 \x01(\x02\"\x89\x01\n\rGTConeSetting\x12\x10\n\x08segments\x18\x01 \x01(\x05\x12\x14\n\x0cvertical_fov\x18\x02 \x01(\x02\x12\x16\n\x0ehorizontal_fov\x18\x03 \x01(\x02\x12\x11\n\trange_min\x18\x04 \x01(\x02\x12\x11\n\trange_max\x18\x05 \x01(\x02\x12\x12\n\nresolution\x18\x07 \x01(\x05\"\xf4\x01\n\x0fGTSensorSetting\x12.\n\nshape_type\x18\x01 \x01(\x0e\x32\x1a.morai_sim_api.GTShapeType\x12:\n\x10\x63ylinder_setting\x18\x02 \x01(\x0b\x32 .morai_sim_api.GTCylinderSetting\x12\x32\n\x0c\x63one_setting\x18\x03 \x01(\x0b\x32\x1c.morai_sim_api.GTConeSetting\x12.\n\x06\x66ilter\x18\x04 \x03(\x0e\x32\x1e.morai_sim_api.ObstacleObjType\x12\x11\n\tmax_count\x18\x05 \x01(\x05\"\x83\x01\n\x14SetGroundTruthSensor\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12\x11\n\tsensor_id\x18\x02 \x01(\t\x12\x32\n\ngt_setting\x18\x03 \x01(\x0b\x32\x1e.morai_sim_api.GTSensorSetting\x12\x11\n\tviz_range\x18\x04 \x01(\x08\"Q\n\x18SetGroundTruthSensorList\x12\x35\n\x08req_list\x18\x01 \x03(\x0b\x32#.morai_sim_api.SetGroundTruthSensor\"4\n\x0cTargetSensor\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12\x11\n\tsensor_id\x18\x02 \x01(\t\"A\n\x10TargetSensorList\x12-\n\x08req_list\x18\x01 \x03(\x0b\x32\x1b.morai_sim_api.TargetSensor\"\xf6\x01\n\x0cObjectStatus\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0f\n\x07heading\x18\x04 \x01(\x01\x12(\n\x08position\x18\x05 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12,\n\x0c\x61\x63\x63\x65leration\x18\x06 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12(\n\x08velocity\x18\x07 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12$\n\x04size\x18\x08 \x01(\x0b\x32\x16.morai_sim_api.Vector3\"I\n\x0bGroundTruth\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12)\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x1b.morai_sim_api.ObjectStatus\"?\n\x0fGroundTruthList\x12,\n\x08res_list\x18\x01 \x03(\x0b\x32\x1a.morai_sim_api.GroundTruth\"V\n\nSaveSensor\x12\x1b\n\x13is_custom_file_name\x18\x01 \x01(\x08\x12\x19\n\x11\x63ustrom_file_name\x18\x02 \x01(\t\x12\x10\n\x08\x66ile_dir\x18\x03 \x01(\t*\xfb\x01\n\nSensorType\x12\x14\n\x10SENSOR_TYPE_NONE\x10\x00\x12\x16\n\x12SENSOR_TYPE_CAMERA\x10\x01\x12\x17\n\x13SENSOR_TYPE_LIDAR3D\x10\x02\x12\x17\n\x13SENSOR_TYPE_LIDAR2D\x10\x03\x12\x13\n\x0fSENSOR_TYPE_GPS\x10\x04\x12\x13\n\x0fSENSOR_TYPE_IMU\x10\x05\x12\x15\n\x11SENSOR_TYPE_RADAR\x10\x06\x12\x1a\n\x16SENSOR_TYPE_ULTRASONIC\x10\x07\x12\x1b\n\x17SENSOR_TYPE_GROUNDTRUTH\x10\x08\x12\x13\n\x0fSENSOR_TYPE_END\x10\t*]\n\x0bGTShapeType\x12\x12\n\x0eGT_SHAPE_BEGIN\x10\x00\x12\x15\n\x11GT_SHAPE_Cylinder\x10\x01\x12\x11\n\rGT_SHAPE_Cone\x10\x02\x12\x10\n\x0cGT_SHAPE_END\x10\x03*\xb8\x03\n\x0fObstacleObjType\x12\x14\n\x10OBSTACL_OBJ_NONE\x10\x00\x12\x17\n\x13OBSTACL_OBJ_VEHICLE\x10\x01\x12\x1a\n\x16OBSTACL_OBJ_PEDESTRIAN\x10\x02\x12\x1a\n\x16OBSTACL_OBJ_EGOVEHICLE\x10\x03\x12\x18\n\x14OBSTACL_OBJ_OBSTACLE\x10\x04\x12\x1a\n\x16OBSTACL_OBJ_SPAWNPOINT\x10\x05\x12\x1c\n\x18OBSTACL_OBJ_MOVINGOBJECT\x10\x06\x12\x1d\n\x19OBSTACL_OBJ_EXTRAOBSTACLE\x10\x07\x12\x1f\n\x1bOBSTACL_OBJ_SSAFYSPAWNPOINT\x10\x08\x12 \n\x1cOBSTACL_OBJ_DESTINATIONPOINT\x10\t\x12\x1d\n\x19OBSTACL_OBJ_STOPOVERPOINT\x10\n\x12\x1d\n\x19OBSTACL_OBJ_PEDSPAWNPOINT\x10\x0b\x12\x1a\n\x16OBSTACL_OBJ_SHADEDAREA\x10\x0c\x12\x19\n\x15OBSTACL_OBJ_MAPOBJECT\x10\r\x12\x13\n\x0fOBSTACL_OBJ_MAX\x10\x0e\x32\xf7\x03\n\x0bMoraiSensor\x12R\n\tAddSensor\x12 .morai_sim_api.SensorSettingList\x1a!.morai_sim_api.SensorResponseList\"\x00\x12T\n\x0cRemoveSensor\x12\x1f.morai_sim_api.RemoveSensorList\x1a!.morai_sim_api.SensorResponseList\"\x00\x12X\n\x14SetGroundTruthSensor\x12\'.morai_sim_api.SetGroundTruthSensorList\x1a\x15.morai_sim_api.Result\"\x00\x12W\n\x12GetGroundTruthData\x12\x1f.morai_sim_api.TargetSensorList\x1a\x1e.morai_sim_api.GroundTruthList\"\x00\x12\x44\n\x0eSaveSensorData\x12\x19.morai_sim_api.SaveSensor\x1a\x15.morai_sim_api.Result\"\x00\x12\x45\n\x0eLoadSensorFile\x12\x1a.morai_sim_api.StringValue\x1a\x15.morai_sim_api.Result\"\x00\x42&H\x01\xaa\x02!CoreLib.Grpc.Protobuf.MoraiSimAPIb\x06proto3')

_SENSORTYPE = DESCRIPTOR.enum_types_by_name['SensorType']
SensorType = enum_type_wrapper.EnumTypeWrapper(_SENSORTYPE)
_GTSHAPETYPE = DESCRIPTOR.enum_types_by_name['GTShapeType']
GTShapeType = enum_type_wrapper.EnumTypeWrapper(_GTSHAPETYPE)
_OBSTACLEOBJTYPE = DESCRIPTOR.enum_types_by_name['ObstacleObjType']
ObstacleObjType = enum_type_wrapper.EnumTypeWrapper(_OBSTACLEOBJTYPE)
SENSOR_TYPE_NONE = 0
SENSOR_TYPE_CAMERA = 1
SENSOR_TYPE_LIDAR3D = 2
SENSOR_TYPE_LIDAR2D = 3
SENSOR_TYPE_GPS = 4
SENSOR_TYPE_IMU = 5
SENSOR_TYPE_RADAR = 6
SENSOR_TYPE_ULTRASONIC = 7
SENSOR_TYPE_GROUNDTRUTH = 8
SENSOR_TYPE_END = 9
GT_SHAPE_BEGIN = 0
GT_SHAPE_Cylinder = 1
GT_SHAPE_Cone = 2
GT_SHAPE_END = 3
OBSTACL_OBJ_NONE = 0
OBSTACL_OBJ_VEHICLE = 1
OBSTACL_OBJ_PEDESTRIAN = 2
OBSTACL_OBJ_EGOVEHICLE = 3
OBSTACL_OBJ_OBSTACLE = 4
OBSTACL_OBJ_SPAWNPOINT = 5
OBSTACL_OBJ_MOVINGOBJECT = 6
OBSTACL_OBJ_EXTRAOBSTACLE = 7
OBSTACL_OBJ_SSAFYSPAWNPOINT = 8
OBSTACL_OBJ_DESTINATIONPOINT = 9
OBSTACL_OBJ_STOPOVERPOINT = 10
OBSTACL_OBJ_PEDSPAWNPOINT = 11
OBSTACL_OBJ_SHADEDAREA = 12
OBSTACL_OBJ_MAPOBJECT = 13
OBSTACL_OBJ_MAX = 14


_SENSORSETTING = DESCRIPTOR.message_types_by_name['SensorSetting']
_SENSORSETTINGLIST = DESCRIPTOR.message_types_by_name['SensorSettingList']
_REMOVESENSOR = DESCRIPTOR.message_types_by_name['RemoveSensor']
_REMOVESENSORLIST = DESCRIPTOR.message_types_by_name['RemoveSensorList']
_SENSORRESPONSE = DESCRIPTOR.message_types_by_name['SensorResponse']
_SENSORRESPONSELIST = DESCRIPTOR.message_types_by_name['SensorResponseList']
_GTCYLINDERSETTING = DESCRIPTOR.message_types_by_name['GTCylinderSetting']
_GTCONESETTING = DESCRIPTOR.message_types_by_name['GTConeSetting']
_GTSENSORSETTING = DESCRIPTOR.message_types_by_name['GTSensorSetting']
_SETGROUNDTRUTHSENSOR = DESCRIPTOR.message_types_by_name['SetGroundTruthSensor']
_SETGROUNDTRUTHSENSORLIST = DESCRIPTOR.message_types_by_name['SetGroundTruthSensorList']
_TARGETSENSOR = DESCRIPTOR.message_types_by_name['TargetSensor']
_TARGETSENSORLIST = DESCRIPTOR.message_types_by_name['TargetSensorList']
_OBJECTSTATUS = DESCRIPTOR.message_types_by_name['ObjectStatus']
_GROUNDTRUTH = DESCRIPTOR.message_types_by_name['GroundTruth']
_GROUNDTRUTHLIST = DESCRIPTOR.message_types_by_name['GroundTruthList']
_SAVESENSOR = DESCRIPTOR.message_types_by_name['SaveSensor']
SensorSetting = _reflection.GeneratedProtocolMessageType('SensorSetting', (_message.Message,), {
  'DESCRIPTOR' : _SENSORSETTING,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SensorSetting)
  })
_sym_db.RegisterMessage(SensorSetting)

SensorSettingList = _reflection.GeneratedProtocolMessageType('SensorSettingList', (_message.Message,), {
  'DESCRIPTOR' : _SENSORSETTINGLIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SensorSettingList)
  })
_sym_db.RegisterMessage(SensorSettingList)

RemoveSensor = _reflection.GeneratedProtocolMessageType('RemoveSensor', (_message.Message,), {
  'DESCRIPTOR' : _REMOVESENSOR,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.RemoveSensor)
  })
_sym_db.RegisterMessage(RemoveSensor)

RemoveSensorList = _reflection.GeneratedProtocolMessageType('RemoveSensorList', (_message.Message,), {
  'DESCRIPTOR' : _REMOVESENSORLIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.RemoveSensorList)
  })
_sym_db.RegisterMessage(RemoveSensorList)

SensorResponse = _reflection.GeneratedProtocolMessageType('SensorResponse', (_message.Message,), {
  'DESCRIPTOR' : _SENSORRESPONSE,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SensorResponse)
  })
_sym_db.RegisterMessage(SensorResponse)

SensorResponseList = _reflection.GeneratedProtocolMessageType('SensorResponseList', (_message.Message,), {
  'DESCRIPTOR' : _SENSORRESPONSELIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SensorResponseList)
  })
_sym_db.RegisterMessage(SensorResponseList)

GTCylinderSetting = _reflection.GeneratedProtocolMessageType('GTCylinderSetting', (_message.Message,), {
  'DESCRIPTOR' : _GTCYLINDERSETTING,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GTCylinderSetting)
  })
_sym_db.RegisterMessage(GTCylinderSetting)

GTConeSetting = _reflection.GeneratedProtocolMessageType('GTConeSetting', (_message.Message,), {
  'DESCRIPTOR' : _GTCONESETTING,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GTConeSetting)
  })
_sym_db.RegisterMessage(GTConeSetting)

GTSensorSetting = _reflection.GeneratedProtocolMessageType('GTSensorSetting', (_message.Message,), {
  'DESCRIPTOR' : _GTSENSORSETTING,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GTSensorSetting)
  })
_sym_db.RegisterMessage(GTSensorSetting)

SetGroundTruthSensor = _reflection.GeneratedProtocolMessageType('SetGroundTruthSensor', (_message.Message,), {
  'DESCRIPTOR' : _SETGROUNDTRUTHSENSOR,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SetGroundTruthSensor)
  })
_sym_db.RegisterMessage(SetGroundTruthSensor)

SetGroundTruthSensorList = _reflection.GeneratedProtocolMessageType('SetGroundTruthSensorList', (_message.Message,), {
  'DESCRIPTOR' : _SETGROUNDTRUTHSENSORLIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SetGroundTruthSensorList)
  })
_sym_db.RegisterMessage(SetGroundTruthSensorList)

TargetSensor = _reflection.GeneratedProtocolMessageType('TargetSensor', (_message.Message,), {
  'DESCRIPTOR' : _TARGETSENSOR,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.TargetSensor)
  })
_sym_db.RegisterMessage(TargetSensor)

TargetSensorList = _reflection.GeneratedProtocolMessageType('TargetSensorList', (_message.Message,), {
  'DESCRIPTOR' : _TARGETSENSORLIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.TargetSensorList)
  })
_sym_db.RegisterMessage(TargetSensorList)

ObjectStatus = _reflection.GeneratedProtocolMessageType('ObjectStatus', (_message.Message,), {
  'DESCRIPTOR' : _OBJECTSTATUS,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.ObjectStatus)
  })
_sym_db.RegisterMessage(ObjectStatus)

GroundTruth = _reflection.GeneratedProtocolMessageType('GroundTruth', (_message.Message,), {
  'DESCRIPTOR' : _GROUNDTRUTH,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GroundTruth)
  })
_sym_db.RegisterMessage(GroundTruth)

GroundTruthList = _reflection.GeneratedProtocolMessageType('GroundTruthList', (_message.Message,), {
  'DESCRIPTOR' : _GROUNDTRUTHLIST,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GroundTruthList)
  })
_sym_db.RegisterMessage(GroundTruthList)

SaveSensor = _reflection.GeneratedProtocolMessageType('SaveSensor', (_message.Message,), {
  'DESCRIPTOR' : _SAVESENSOR,
  '__module__' : 'morai_sensor_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.SaveSensor)
  })
_sym_db.RegisterMessage(SaveSensor)

_MORAISENSOR = DESCRIPTOR.services_by_name['MoraiSensor']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\001\252\002!CoreLib.Grpc.Protobuf.MoraiSimAPI'
  _SENSORTYPE._serialized_start=1837
  _SENSORTYPE._serialized_end=2088
  _GTSHAPETYPE._serialized_start=2090
  _GTSHAPETYPE._serialized_end=2183
  _OBSTACLEOBJTYPE._serialized_start=2186
  _OBSTACLEOBJTYPE._serialized_end=2626
  _SENSORSETTING._serialized_start=56
  _SENSORSETTING._serialized_end=222
  _SENSORSETTINGLIST._serialized_start=224
  _SENSORSETTINGLIST._serialized_end=291
  _REMOVESENSOR._serialized_start=293
  _REMOVESENSOR._serialized_end=345
  _REMOVESENSORLIST._serialized_start=347
  _REMOVESENSORLIST._serialized_end=412
  _SENSORRESPONSE._serialized_start=414
  _SENSORRESPONSE._serialized_end=485
  _SENSORRESPONSELIST._serialized_start=487
  _SENSORRESPONSELIST._serialized_end=556
  _GTCYLINDERSETTING._serialized_start=558
  _GTCYLINDERSETTING._serialized_end=632
  _GTCONESETTING._serialized_start=635
  _GTCONESETTING._serialized_end=772
  _GTSENSORSETTING._serialized_start=775
  _GTSENSORSETTING._serialized_end=1019
  _SETGROUNDTRUTHSENSOR._serialized_start=1022
  _SETGROUNDTRUTHSENSOR._serialized_end=1153
  _SETGROUNDTRUTHSENSORLIST._serialized_start=1155
  _SETGROUNDTRUTHSENSORLIST._serialized_end=1236
  _TARGETSENSOR._serialized_start=1238
  _TARGETSENSOR._serialized_end=1290
  _TARGETSENSORLIST._serialized_start=1292
  _TARGETSENSORLIST._serialized_end=1357
  _OBJECTSTATUS._serialized_start=1360
  _OBJECTSTATUS._serialized_end=1606
  _GROUNDTRUTH._serialized_start=1608
  _GROUNDTRUTH._serialized_end=1681
  _GROUNDTRUTHLIST._serialized_start=1683
  _GROUNDTRUTHLIST._serialized_end=1746
  _SAVESENSOR._serialized_start=1748
  _SAVESENSOR._serialized_end=1834
  _MORAISENSOR._serialized_start=2629
  _MORAISENSOR._serialized_end=3132
# @@protoc_insertion_point(module_scope)



import gzip
import base64

def decompress(compressed, base64_encoded = True):
    source = compressed
    if base64_encoded:
        source = base64.standard_b64decode(compressed)

    return gzip.decompress(source).decode('utf-8')



import os, sys

current_path = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.normpath(os.path.join(current_path, './')))
sys.path.append(os.path.normpath(os.path.join(current_path, '../')))

import grpc
import google.protobuf.empty_pb2
import protobuf.morai_actor_pb2
import protobuf.morai_actor_pb2_grpc
import protobuf.morai_mgeo_pb2
import protobuf.morai_mgeo_pb2_grpc
import protobuf.morai_sim_info_pb2
import protobuf.morai_sim_info_pb2_grpc
import protobuf.morai_simulation_pb2
import protobuf.morai_simulation_pb2_grpc
import protobuf.morai_sensor_pb2
import protobuf.morai_sensor_pb2_grpc
import protobuf.morai_type_pb2

import threading
from common_utils.compress_utils import decompress

MAX_MESSAGE_LENGTH = 33554432 # 32mb
DEFAULT_MORAI_SIM_IP = '127.0.0.1'
DEFAULT_MORAI_SIM_PORT = 7780

class SimAdapter():
    def __init__(self):
        self.__ip = DEFAULT_MORAI_SIM_IP
        self.__port = DEFAULT_MORAI_SIM_PORT
        self.__channel = None
        self.__actor_stub = None
        self.__mgeo_stub = None
        self.__sim_info_stub = None
        self.__simulation_stub = None
        self.__sensor_stub = None
        self.__actor_state_thread = None
        self.__on_actor_state_received_callback = None
        self.__actor_states = protobuf.morai_actor_pb2.ActorStates()
        self.__actor_state_recevier_stop_flag = False
        self.__actor_states_lock = threading.Lock()


    def get_ip(self):
        return self.__ip


    def get_port(self):
        return self.__port


    def is_connected(self):
        return False if (self.__channel == None) else True


    def connect(self, ip=DEFAULT_MORAI_SIM_IP, port=DEFAULT_MORAI_SIM_PORT):
        if self.is_connected():
            self.disconnect()

        self.__ip = ip
        self.__port = port

        self.__channel = grpc.insecure_channel(
            f'{self.__ip}:{self.__port}',
            options=[
                ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH),
            ]
        )
        # self.channel = grpc.insecure_channel(f'{self.ip}:{self.port}')
        self.__actor_stub = protobuf.morai_actor_pb2_grpc.MoraiActorStub(self.__channel)
        self.__mgeo_stub = protobuf.morai_mgeo_pb2_grpc.MoraiMgeoStub(self.__channel)
        self.__sim_info_stub = protobuf.morai_sim_info_pb2_grpc.MoraiSimInfoStub(self.__channel)
        self.__simulation_stub = protobuf.morai_simulation_pb2_grpc.MoraiSimulationStub(self.__channel)
        self.__sensor_stub = protobuf.morai_sensor_pb2_grpc.MoraiSensorStub(self.__channel)
        

    def disconnect(self):
        if not self.is_connected():
            return

        try:
            self.__channel.close()
            self.__channel = None

            self.__actor_stub = None
            self.__mgeo_stub = None
            self.__sim_info_stub = None
            self.__simulation_stub = None

            self.__actor_state_recevier_stop_flag = True
            if self.__actor_state_thread != None:
                self.__actor_state_thread.join()

        except BaseException as e:
            print(e)


    def initialize(self):
        try:
            request = google.protobuf.empty_pb2.Empty()
            self.__simulation_stub.Initialize(request)
        except BaseException as e:
            print(f'initialize failed : {e}')


    def get_simulator_version(self):
        simulator_version = ""
        try:
            request = google.protobuf.empty_pb2.Empty()
            response = self.__sim_info_stub.GetSimulatorVersion(request)
            simulator_version = response.value
        except Exception as e:
            print(f'get_simulator_version failed : {e}')
        return simulator_version


    def get_available_maps(self):
        available_maps = []
        try:
            request = google.protobuf.empty_pb2.Empty()
            response = self.__sim_info_stub.GetAvailableMaps(request)
            available_maps = response.values
        except Exception as e:
            print(f'get_available_maps failed : {e}')

        return available_maps


    def get_data_path(self):
        data_path = ''
        try:
            request = google.protobuf.empty_pb2.Empty()
            response = self.__sim_info_stub.GetDataPath(request)
            data_path = response.value
        except Exception as e:
            print(f'get_Data_path failed : {e}')

        return data_path


    def get_actor_types(self):
        actor_types = [
            protobuf.morai_actor_pb2.ACTOR_TYPE_UNKNOWN,
            protobuf.morai_actor_pb2.ACTOR_TYPE_VEHICLE,
            protobuf.morai_actor_pb2.ACTOR_TYPE_PEDESTRIAN,
            protobuf.morai_actor_pb2.ACTOR_TYPE_OBSTACLE]

        return actor_types


    def set_ego(self, ego_ctrl_cmd):
        try:
            response = self.__actor_stub.SetEgo(ego_ctrl_cmd)
            return response.success
        except Exception as e:
            print(f'ego ctrl failed : {e}')


    def load_world(self, map_name, ego_vehicle_model):
        try:
            request = protobuf.morai_simulation_pb2.LoadWorldParam()
            request.map_name = map_name
            request.ego_vehicle_model = ego_vehicle_model
            request.wait = True
            response = self.__simulation_stub.LoadWorld(request)
            if not response.success:
                print(f'load world result : {response.description}')
        except Exception as e:
            print(f'load_map failed : {e}')
            return False

        return True


    def ray_cast(self, raycast_param):
        occlusion_dist = -1.0
        try:
            response = self.__simulation_stub.RayCast(raycast_param)
            occlusion_dist = response.value
        except Exception as e:
            print(f'ray_cast failed : {e}')

        return occlusion_dist


    def spawn_actor(self, spawn_actor_param):        
        actor_responses = None
        try:            
            actor_responses = self.__actor_stub.SpawnActor(spawn_actor_param)
        except Exception as e:
            print(f'spawn_actor failed : {e}')

        return actor_responses


    def vehicle_control(self, vehicle_ctrl_cmd):
        result = False
        try:
            response = self.__actor_stub.VehicleControl(vehicle_ctrl_cmd)
            result = response.success
        except Exception as e:
            print(f'vehicle_control failed : {e}')

        return result


    def destroy_actor(self, actor_info):
        try:
            response = self.__actor_stub.DestroyActor(actor_info)
            # print("destroy_actor result {}".format(response.success))
        except Exception as e:
            print(f'destroy_actor failed : {e}')


    def destroy_all_actors(self):
        try:
            response = self.__actor_stub.DestroyAllActors(google.protobuf.empty_pb2.Empty())
            # print(f'destroy_all_actors result {response.success}')
        except Exception as e:
            print(f'destroy_all_actors failed : {e}')


    def get_timestamp(self):
        sim_timestamp = -1
        try:
            response = self.__sim_info_stub.GetTimestamp(google.protobuf.empty_pb2.Empty())
            sim_timestamp = response.value
        except Exception as e:
            print("get_timestamp failed : ", e)

        return sim_timestamp


    def get_weather(self):
        weather = protobuf.morai_simulation_pb2.WEATHER_TYPE_SUNNY        
        try:
            response = self.__simulation_stub.GetWeather(google.protobuf.empty_pb2.Empty())
            weather = response.weather_type
        except Exception as e:
            print("get_weather failed : ", e)
        
        return weather


    def set_weather(self, weather_type):
        request = protobuf.morai_simulation_pb2.WeatherParam()
        request.weather_type = weather_type
        try:
            self.__simulation_stub.SetWeather(request)
        except Exception as e:
            print("set_weather failed : ", e)


    def pause(self):
        result = False
        try:
            response = self.__simulation_stub.Pause(google.protobuf.empty_pb2.Empty())
            result = response.success
        except Exception as e:
            print("pause failed : ", e)
            
        return result


    def resume(self):
        result = False
        try:
            response = self.__simulation_stub.Resume(google.protobuf.empty_pb2.Empty())
            result = response.success
        except Exception as e:
            print("resume failed : ", e)
            
        return result


    def is_synchronous_mode(self):
        enabled = False
        try:
            result = self.__simulation_stub.IsSynchronousMode(google.protobuf.empty_pb2.Empty())
            enabled = result.value
        except Exception as e:
            print("is_synchronous_mode failed : ", e)
            
        return enabled


    def set_synchronous_mode(self, enabled):
        result = False
        try:
            request = protobuf.morai_type_pb2.BoolValue()
            request.value = enabled

            response = self.__simulation_stub.SetSynchronousMode(request)
            result = response.success
        except Exception as e:
            print("set_synchronous_mode failed : ", e)

        return result


    def tick(self, elapse_time):
        result = False
        try:
            request = protobuf.morai_type_pb2.Int32Value()
            request.value = elapse_time

            response = self.__simulation_stub.Tick(request)
            result = response.value
        except Exception as e:
            print("tick failed : ", e)

        return result


    def wait_for_tick(self):
        sim_timestamp = -1
        try:
            request = google.protobuf.empty_pb2.Empty()
            response = self.__simulation_stub.WaitForTick(request)
            sim_timestamp = response.value
        except Exception as e:
            print("wait_for_tick failed : ", e)

        return sim_timestamp


    def get_mgeo(self, map_name):
        mgeo_data = None
        try:
            request = protobuf.morai_mgeo_pb2.MapInfo()
            request.map_name = map_name

            mgeo_data = self.__mgeo_stub.GetMGeo(request)
            self.decompress_mgeo(mgeo_data)

        except Exception as e:
            print("get_mgeo failed : ", e)

        return mgeo_data


    def start_actor_state_receiver(self, on_received_callback = None):
        self.__actor_state_recevier_stop_flag = False
        self.__on_actor_state_received_callback = on_received_callback
        self.__actor_state_thread = threading.Thread(target=self.receive_actor_state, args=(lambda: self.__actor_state_recevier_stop_flag, ))
        self.__actor_state_thread.start()


    def receive_actor_state(self, stop):
        self.__actor_state_channel = grpc.insecure_channel(f'{self.__ip}:{self.__port}')
        self.__actor_state_stub = protobuf.morai_actor_pb2_grpc.MoraiActorStub(self.__actor_state_channel)

        result = None
        while result is None:
            try:
                actor_state_stream = self.__actor_state_stub.GetActorStateStream(google.protobuf.empty_pb2.Empty())
                for actor_states in actor_state_stream:
                    self.__actor_states_lock.acquire()
                    self.__actor_states.CopyFrom(actor_states)
                    self.__actor_states_lock.release()
                    if self.__on_actor_state_received_callback != None:
                        self.__on_actor_state_received_callback()

                    if stop():
                        actor_state_stream.cancel()
                        break
                result = True
                break
            except:
                print("actor_state_stream is empty. Trying again")
                pass


    def get_actor_state(self, actor_info):
            actor_state = None
            try:
                actor_state = self.__actor_stub.GetActorState(actor_info)
            except Exception as e:
                print(f'get_actor_state failed : {e}')

            return actor_state

    def set_actor_property(self, actor_property):
        success = False
        try:
            response = self.__actor_stub.SetActorProperty(actor_property)
            success = response.success
        except Exception as e:
            print(f'set actor property failed : {e}')

        return success


    def get_actor_network_config(self, actor_info):
        response = None
        try:
            response = self.__actor_stub.GetActorNetworkSetting(actor_info)
        except Exception as e:
            print(f'get actor network config failed : {e}')
        
        return response


    def get_traffic_light_info_by_link(self, link_info):
        tl_info = None
        try:
            tl_info = self.__actor_stub.GetTrafficLightInfoByLink(link_info)
        except BaseException as e:
            print(f'get_traffic_light_info_by_link failed : {e}')

        return tl_info


    def get_traffic_light_info_by_uid(self, traffic_light_id):
        tl_info = None
        try:
            tl_info = self.__actor_stub.GetTrafficLightInfoByUid(traffic_light_id)
        except BaseException as e:
            print(f'get_traffic_light_info_by_uid failed : {e}')

        return tl_info


    def get_intscn_tl_info(self, intscn_id):
        tl_info_list = None
        try:
            tl_info_list = self.__actor_stub.GetIntscnTLInfo(intscn_id)
        except BaseException as e:
            print(f'get_intscn_tl_info failed : {e}')

        return tl_info_list


    def set_traffic_light_state(self, traffic_light_states):
        actor_responses = None
        try:
            actor_responses = self.__actor_stub.SetTrafficLightState(traffic_light_states)
        except BaseException as e:
            print(f'set traffic light state failed : {e}')

        return actor_responses


    def set_intersection_state(self, intersection_param_list):
        try:
            response = self.__actor_stub.SetIntersectionState(intersection_param_list)
            return response
        except BaseException as e:
            return

    def set_intersection_schedule(self, intersection_schedule_list):
        try:
            response = self.__actor_stub.SetIntersectionSchedule(intersection_schedule_list)
            return response
        except BaseException as e:
            return

    def set_vehicle_route(self, vehicle_route):
        try:
            response = self.__actor_stub.SetVehicleRoute(vehicle_route)
            print(f'route response : {response.description}')
            return response
        except BaseException as e:
            print(f'route exception : {e}')
            return

    def set_vehicle_destination(self, vehicle_destination):
        try:
            response = self.__actor_stub.SetVehicleDestination(vehicle_destination)
            print(f'set vehicle destination : {response.description}')
            return response
        except BaseException as e:
            print(f'set vehicle destination exception : {e}')

    def add_sensor(self, sensor_setting_list):
        try:
            response = self.__sensor_stub.AddSensor(sensor_setting_list)
            return response
        except BaseException as e:
            print(f'add sensor exception : {e}')

    def remove_sensor(self, remove_sensor_list):
        try:
            response = self.__sensor_stub.RemoveSensor(remove_sensor_list)
            return response
        except BaseException as e:
            print(f'remove sensor exception : {e}')

    def set_gt_sensor(self, set_gt_sensor_list):
        try:
            response = self.__sensor_stub.SetGroundTruthSensor(set_gt_sensor_list)
            return response
        except BaseException as e:
            print(f'set gt sensor exception : {e}')        

    def get_gt_data(self, target_sensor_list):
        try:
            response = self.__sensor_stub.GetGroundTruthData(target_sensor_list)
            return response
        except BaseException as e:
            print(f'get gt data exception : {e}')  

    def save_sensor_data(self, save_sensor_data):
        try:
            self.__sensor_stub.SaveSensorData(save_sensor_data)
        except BaseException as e:
            print(f'save sensor data exception : {e}')
            
    def load_sensor_file(self, sensor_filename):
        try:
            request = protobuf.morai_type_pb2.StringValue()
            request.value = sensor_filename
            response = self.__sensor_stub.LoadSensorFile(request)
            return response
        except BaseException as e:
            print(f'load sensor file exception : {e}')

    def decompress_mgeo(self, mgeo_data):
        mgeo_data.global_info = decompress(mgeo_data.global_info)
        mgeo_data.lane_marking_set = decompress(mgeo_data.lane_marking_set)
        mgeo_data.lane_node_set = decompress(mgeo_data.lane_node_set)
        mgeo_data.linke_set = decompress(mgeo_data.linke_set)
        mgeo_data.node_set = decompress(mgeo_data.node_set)
        mgeo_data.traffic_light_set = decompress(mgeo_data.traffic_light_set)
        mgeo_data.intersection_controller_set = decompress(mgeo_data.intersection_controller_set)
        mgeo_data.intersection_controller_data = decompress(mgeo_data.intersection_controller_data)



from moraikit.protobuf.conversions import from_actor_state_msg, to_transform_msg, to_vector3_msg
from moraikit.types import ObjectParams, Transform, Vector3
from moraikit.sim_adapter import SimAdapter
import moraikit.protobuf.morai_actor_pb2
import protobuf.morai_type_pb2
import protobuf.morai_actor_pb2
import google.protobuf.empty_pb2
from protobuf.morai_sensor_pb2 import SENSOR_TYPE_CAMERA, SaveSensor, SensorSetting, SensorSettingList, RemoveSensor, RemoveSensorList
from copy import deepcopy as dc

DEFAULT_MORAI_SIM_IP = '127.0.0.1'
DEFAULT_MORAI_SIM_PORT = 7780

class SensorApi(SimAdapter):

    def connect(self, ip=DEFAULT_MORAI_SIM_IP, port=DEFAULT_MORAI_SIM_PORT):
        return super().connect(ip, port)      

    def disconnect(self):
        return super().disconnect()

    def initialize(self):
        return super().initialize()

    def get_ip(self):
        return super().get_ip()

    def get_port(self):
        return super().get_port()

    def is_connected(self):
        return super().is_connected()

    def get_simulator_version(self):
        return super().get_simulator_version()

    def get_available_objects(self):
        available_objects = []
        try:
            request = google.protobuf.empty_pb2.Empty()
            response = super().__sim_info_stub.GetAvailableObject(request)
            available_objects = response.values
        except Exception as e:
            print(f'get_available_objects failed : {e}')
        return available_objects

    def get_available_maps(self):
        return super().get_available_maps()
    
    def get_data_path(self):
        return super().get_data_path()

    def get_ego_transform(self):
        return self.get_actor_state('0').transform

    def load_world(self, map_name, ego_name='2017_Kia_Niro(HEV)'):
        return super().load_world(map_name, ego_name)

    # def spawn_object(self, transform, obj_params=ObjectParams()):
    #     spawn_actor_params = protobuf.morai_actor_pb2.SpawnActorParams()
    #     param = protobuf.morai_actor_pb2.SpawnActorParam()        
    #     param.actor_info.actor_id = '-1'
    #     param.actor_info.actor_type = moraikit.protobuf.morai_actor_pb2.ACTOR_TYPE_OBSTACLE
        
    #     to_transform_msg(param.transform, transform)

    #     if obj_params.scale is None:
    #         obj_params.scale = Vector3(1.0, 1.0, 1.0)
    #     _obj_params = dc(obj_params)            
    #     _obj_params.scale.x = obj_params.scale.y
    #     _obj_params.scale.y = obj_params.scale.z
    #     _obj_params.scale.z = obj_params.scale.x       
    #     to_vector3_msg(param.obstacle.scale, _obj_params.scale)
    #     param.obstacle.obstacle_name = obj_params.object_name

    #     spawn_actor_params.params.append(param)
    #     response = super().spawn_actor(spawn_actor_params)
        
    #     return self.extract_role_name(response)

    def spawn_object(self, distance, obj_params=ObjectParams()):
        spawn_actor_params = protobuf.morai_actor_pb2.SpawnActorParams()
        param = protobuf.morai_actor_pb2.SpawnActorParam()        
        param.actor_info.actor_id = '-1'
        param.actor_info.actor_type = moraikit.protobuf.morai_actor_pb2.ACTOR_TYPE_OBSTACLE
        
        _ego_transform = self.get_ego_transform()
        object_transform = _ego_transform.get_transform_to(distance, obj_params)
        transform = _ego_transform.transform(object_transform)

        to_transform_msg(param.transform, transform)

        if obj_params.scale is None:
            obj_params.scale = Vector3(1.0, 1.0, 1.0)
        _obj_params = dc(obj_params)            
        _obj_params.scale.x = obj_params.scale.y
        _obj_params.scale.y = obj_params.scale.z
        _obj_params.scale.z = obj_params.scale.x       
        to_vector3_msg(param.obstacle.scale, _obj_params.scale)
        param.obstacle.obstacle_name = obj_params.object_name

        spawn_actor_params.params.append(param)
        response = super().spawn_actor(spawn_actor_params)
        
        return self.extract_role_name(response)

    def spawn_object_from_camera(self, obj_id = '-1', camera_transform:Transform=None, obj_transform:Transform=None, obj_params=ObjectParams()):
        spawn_actor_params = protobuf.morai_actor_pb2.SpawnActorParams()
        param = protobuf.morai_actor_pb2.SpawnActorParam()        
        param.actor_info.actor_id = obj_id
        param.actor_info.actor_type = moraikit.protobuf.morai_actor_pb2.ACTOR_TYPE_OBSTACLE
        ego_transform = self.get_ego_transform()
        transform = ego_transform.transform(camera_transform.transform(obj_transform))
        to_transform_msg(param.transform, transform)

        if obj_params.scale is None:
            obj_params.scale = Vector3(1.0, 1.0, 1.0)
        _obj_params = dc(obj_params)            
        _obj_params.scale.x = obj_params.scale.y
        _obj_params.scale.y = obj_params.scale.z
        _obj_params.scale.z = obj_params.scale.x
        to_vector3_msg(param.obstacle.scale, _obj_params.scale)
        param.obstacle.obstacle_name = obj_params.object_name

        spawn_actor_params.params.append(param)
        response = super().spawn_actor(spawn_actor_params)
        
        return self.extract_role_name(response)

    def remove_object(self, object_id):
        param = protobuf.morai_actor_pb2.SpawnActorParam()        
        param.actor_info.actor_id = object_id
        param.actor_info.actor_type = moraikit.protobuf.morai_actor_pb2.ACTOR_TYPE_OBSTACLE
        
        self.destroy_actor(param.actor_info)
        return

    def set_camera(self, transform:Transform=None):
        # data -> proto
        request = SensorSettingList()
        
        msg = SensorSetting()
        msg.unique_id = '0' # 차량 ID
        msg.sensor_type = SENSOR_TYPE_CAMERA
        msg.position.x = transform.location.x
        msg.position.y = transform.location.y
        msg.position.z = transform.location.z
        msg.rotation.x = transform.rotation.roll
        msg.rotation.y = transform.rotation.pitch
        msg.rotation.z = transform.rotation.yaw

        request.req_list.append(msg)

        response = super().add_sensor(request)
        return response        
        
    def capture_data(self): 
        request = SaveSensor()
        request.is_custom_file_name = False
        super().save_sensor_data(request)

    def capture_data_custom(self, file_name:str="sample", file_dir:str="sample"):
        """파일 위치 및 이름 설정"""
        request = SaveSensor()
        request.is_custom_file_name = True
        request.custrom_file_name = file_name
        request.file_dir = file_dir
        super().save_sensor_data(request)

    def remove_camera(self, sensor_id):
        # data -> proto
        request = RemoveSensorList()
        
        msg = RemoveSensor()
        msg.unique_id = '0'
        msg.sensor_id = str(sensor_id)

        request.req_list.append(msg)
        
        response = super().remove_sensor(request)
        return response

    def load_sensor_preset(self, file_name):
        response = super().load_sensor_file(file_name)
        return response

    def get_actor_state(self, actor_id):
        actor_info = moraikit.protobuf.morai_actor_pb2.ActorInfo()
        actor_info.actor_id = actor_id
        actor_info.actor_type = moraikit.protobuf.morai_actor_pb2.ACTOR_TYPE_VEHICLE
        actor_state = super().get_actor_state(actor_info)
        conv_actor_state = from_actor_state_msg(actor_state)   

        return conv_actor_state


    """ Helper methods """
    def extract_role_name(self, actor_responses):
        role_name = ''
        if actor_responses == None or actor_responses.values[0].result == False:
            print('spawn actor error')
        else:
            role_name = actor_responses.values[0].display_name
        
        return role_name




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: morai_mgeo.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import morai_type_pb2 as morai__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10morai_mgeo.proto\x12\rmorai_sim_api\x1a\x10morai_type.proto\"\x1b\n\x07MapInfo\x12\x10\n\x08map_name\x18\x01 \x01(\t\"\xdb\x01\n\x08MGeoData\x12\x13\n\x0bglobal_info\x18\x01 \x01(\t\x12\x18\n\x10lane_marking_set\x18\x02 \x01(\t\x12\x15\n\rlane_node_set\x18\x03 \x01(\t\x12\x11\n\tlinke_set\x18\x04 \x01(\t\x12\x10\n\x08node_set\x18\x05 \x01(\t\x12\x19\n\x11traffic_light_set\x18\x06 \x01(\t\x12#\n\x1bintersection_controller_set\x18\x07 \x01(\t\x12$\n\x1cintersection_controller_data\x18\x08 \x01(\t\"4\n\x0fGetLinkResponse\x12\x0e\n\x06result\x18\x01 \x01(\x08\x12\x11\n\tunique_id\x18\x02 \x01(\t\"P\n\x13GetLinkResponseList\x12\x0e\n\x06result\x18\x01 \x01(\x08\x12)\n\x08res_list\x18\x02 \x03(\x0b\x32\x17.morai_sim_api.LinkInfo\" \n\x0bVehicleInfo\x12\x11\n\tunique_id\x18\x01 \x01(\t\"]\n\x1dGetVehiclesOnLinkResponseList\x12\x0e\n\x06result\x18\x01 \x01(\x08\x12,\n\x08res_list\x18\x02 \x03(\x0b\x32\x1a.morai_sim_api.VehicleInfo2\xd8\x03\n\tMoraiMgeo\x12<\n\x07GetMGeo\x12\x16.morai_sim_api.MapInfo\x1a\x17.morai_sim_api.MGeoData\"\x00\x12H\n\x0bGetLeftLink\x12\x17.morai_sim_api.LinkInfo\x1a\x1e.morai_sim_api.GetLinkResponse\"\x00\x12I\n\x0cGetRightLink\x12\x17.morai_sim_api.LinkInfo\x1a\x1e.morai_sim_api.GetLinkResponse\"\x00\x12K\n\nGetToLinks\x12\x17.morai_sim_api.LinkInfo\x1a\".morai_sim_api.GetLinkResponseList\"\x00\x12M\n\x0cGetFromLinks\x12\x17.morai_sim_api.LinkInfo\x1a\".morai_sim_api.GetLinkResponseList\"\x00\x12\\\n\x11GetVehiclesOnLink\x12\x17.morai_sim_api.LinkInfo\x1a,.morai_sim_api.GetVehiclesOnLinkResponseList\"\x00\x42&H\x01\xaa\x02!CoreLib.Grpc.Protobuf.MoraiSimAPIb\x06proto3')



_MAPINFO = DESCRIPTOR.message_types_by_name['MapInfo']
_MGEODATA = DESCRIPTOR.message_types_by_name['MGeoData']
_GETLINKRESPONSE = DESCRIPTOR.message_types_by_name['GetLinkResponse']
_GETLINKRESPONSELIST = DESCRIPTOR.message_types_by_name['GetLinkResponseList']
_VEHICLEINFO = DESCRIPTOR.message_types_by_name['VehicleInfo']
_GETVEHICLESONLINKRESPONSELIST = DESCRIPTOR.message_types_by_name['GetVehiclesOnLinkResponseList']
MapInfo = _reflection.GeneratedProtocolMessageType('MapInfo', (_message.Message,), {
  'DESCRIPTOR' : _MAPINFO,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.MapInfo)
  })
_sym_db.RegisterMessage(MapInfo)

MGeoData = _reflection.GeneratedProtocolMessageType('MGeoData', (_message.Message,), {
  'DESCRIPTOR' : _MGEODATA,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.MGeoData)
  })
_sym_db.RegisterMessage(MGeoData)

GetLinkResponse = _reflection.GeneratedProtocolMessageType('GetLinkResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLINKRESPONSE,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GetLinkResponse)
  })
_sym_db.RegisterMessage(GetLinkResponse)

GetLinkResponseList = _reflection.GeneratedProtocolMessageType('GetLinkResponseList', (_message.Message,), {
  'DESCRIPTOR' : _GETLINKRESPONSELIST,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GetLinkResponseList)
  })
_sym_db.RegisterMessage(GetLinkResponseList)

VehicleInfo = _reflection.GeneratedProtocolMessageType('VehicleInfo', (_message.Message,), {
  'DESCRIPTOR' : _VEHICLEINFO,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.VehicleInfo)
  })
_sym_db.RegisterMessage(VehicleInfo)

GetVehiclesOnLinkResponseList = _reflection.GeneratedProtocolMessageType('GetVehiclesOnLinkResponseList', (_message.Message,), {
  'DESCRIPTOR' : _GETVEHICLESONLINKRESPONSELIST,
  '__module__' : 'morai_mgeo_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.GetVehiclesOnLinkResponseList)
  })
_sym_db.RegisterMessage(GetVehiclesOnLinkResponseList)

_MORAIMGEO = DESCRIPTOR.services_by_name['MoraiMgeo']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\001\252\002!CoreLib.Grpc.Protobuf.MoraiSimAPI'
  _MAPINFO._serialized_start=53
  _MAPINFO._serialized_end=80
  _MGEODATA._serialized_start=83
  _MGEODATA._serialized_end=302
  _GETLINKRESPONSE._serialized_start=304
  _GETLINKRESPONSE._serialized_end=356
  _GETLINKRESPONSELIST._serialized_start=358
  _GETLINKRESPONSELIST._serialized_end=438
  _VEHICLEINFO._serialized_start=440
  _VEHICLEINFO._serialized_end=472
  _GETVEHICLESONLINKRESPONSELIST._serialized_start=474
  _GETVEHICLESONLINKRESPONSELIST._serialized_end=567
  _MORAIMGEO._serialized_start=570
  _MORAIMGEO._serialized_end=1042
# @@protoc_insertion_point(module_scope)



import numpy as np
from common_utils.transformation import get_tr, r2rpy, iso2unity, unity2iso
from enum import IntEnum

class GearMode(IntEnum):
    GEAR_MODE_M = 0
    GEAR_MODE_P = 1
    GEAR_MODE_R = 2
    GEAR_MODE_N = 3
    GEAR_MODE_D = 4
    GEAR_MODE_L = 5

class ActorType(IntEnum):    
    ACTOR_TYPE_UNKNOWN = 0
    ACTOR_TYPE_VEHICLE = 1
    ACTOR_TYPE_PEDESTRIAN = 2
    ACTOR_TYPE_OBSTACLE = 3

class ActorPropertyType(IntEnum):
    ACTOR_PROPERTY_TRANSFORM = 0
    ACTOR_PROPERTY_NETWORK = 1
    ACTOR_PROPERTY_PHYSICS = 2
    ACTOR_PROPERTY_AI = 3
    ACTOR_PROPERTY_VEHICLE_EVENT = 4

class NetworkCommType(IntEnum):
    NETWORK_COMM_TYPE_BEGIN = 0
    NETWORK_COMM_TYPE_UDP = 1
    NETWORK_COMM_TYPE_TCP = 2
    NETWORK_COMM_TYPE_ROS = 3
    NETWORK_COMM_TYPE_LCM = 4
    NETWORK_COMM_TYPE_SERIAL = 5
    NETWORK_COMM_TYPE_ROS_AUTO = 6
    NETWORK_COMM_TYPE_APOLLO = 7
    NETWORK_COMM_TYPE_END = 8

class NetworkItemType(IntEnum):
    NETWORK_ITEM_TYPE_BEGIN = 0
    NETWORK_ITEM_TYPE_HOST_IP = 1
    NETWORK_ITEM_TYPE_DESTINATION_IP = 2
    NETWORK_ITEM_TYPE_BRIDGE_IP = 3
    NETWORK_ITEM_TYPE_HOST_PORT = 4
    NETWORK_ITEM_TYPE_DESTINATION_PORT = 5
    NETWORK_ITEM_TYPE_BRIDGE_PORT = 6
    NETWORK_ITEM_TYPE_MESSAGE_TYPE = 7
    NETWORK_ITEM_TYPE_MESSAGE_TOPIC = 8
    NETWORK_ITEM_TYPE_MESSAGE_CHANNEL = 9
    NETWORK_ITEM_TYPE_SERIAL_FIELD = 10
    NETWORK_ITEM_TYPE_SERIAL_PORTNUMBER = 11
    NETWORK_ITEM_TYPE_SERIAL_BAUDRATE = 12
    NETWORK_ITEM_TYPE_SERIAL_PARITY = 13
    NETWORK_ITEM_TYPE_SERIAL_DATABITS = 14
    NETWORK_ITEM_TYPE_SERIAL_STOPBITS = 15
    NETWORK_ITEM_TYPE_SERIAL_FLOWCONTROL = 16
    NETWORK_ITEM_TYPE_SERVCIE_NAME = 17
    NETWORK_ITEM_TYPE_SRV = 18
    NETWORK_ITEM_TYPE_FRAME_ID = 19
    NETWORK_ITEM_TYPE_CHILDFRAME_ID = 20
    NETWORK_ITEM_TYPE_END = 21

class PUBSUBType(IntEnum):
    PUBSUB_TYPE_BEGIN = 0
    PUBSUB_TYPE_Common = 256
    PUBSUB_TYPE_ObjectInfoPublisher = 257
    PUBSUB_TYPE_TLStatusPublisher = 258
    PUBSUB_TYPE_TLCtrlSubscriber = 259
    PUBSUB_TYPE_MultiEgoTransformHandler = 260
    PUBSUB_TYPE_CollisionData = 261
    PUBSUB_TYPE_SensorPosSubscriber = 262
    PUBSUB_TYPE_GlobalIntersectionPublisher = 263
    PUBSUB_TYPE_IntersectionControlSubscriber = 264
    PUBSUB_TYPE_IntersectionStatusPublisher = 265
    PUBSUB_TYPE_SensorSyncDataSubscriber = 266
    PUBSUB_TYPE_TF2Publisher = 267
    PUBSUB_TYPE_VehicleCollisionInfoPublisher = 268
    PUBSUB_TYPE_SyncModeCmdServiceProvider = 269
    PUBSUB_TYPE_SyncModeWaitForTickServiceProvider = 270
    PUBSUB_TYPE_SyncModeInfoPublisher = 271
    PUBSUB_TYPE_SyncModeAddObjectSubscriber = 272
    PUBSUB_TYPE_SyncModeRemoveObjectSubscriber = 273
    PUBSUB_TYPE_SyncModeSetGearServiceProvider = 274
    PUBSUB_TYPE_SyncModeCtrlCmdServiceProvider = 275
    PUBSUB_TYPE_SyncModeSaveSensorDataServiceProvider = 276
    PUBSUB_TYPE_SyncModeMoraiSLServiceProvider = 277
    PUBSUB_TYPE_MandoObjectInfoPublisher = 278
    PUBSUB_TYPE_HILSMoraiInfoPublisher = 279
    PUBSUB_TYPE_FaultStatusPublisher = 280
    PUBSUB_TYPE_FaultStatusVehicleSubscriber = 281
    PUBSUB_TYPE_AutoeverIFSSubscriber = 282
    PUBSUB_TYPE_FaultStatusWheelSubscriber = 283
    PUBSUB_TYPE_FaultStatusSensorSubscriber = 284
    PUBSUB_TYPE_AutoeverInfoSubscriber = 285
    PUBSUB_TYPE_ERP = 512
    PUBSUB_TYPE_ERPInfoPublisher = 513
    PUBSUB_TYPE_ERPSerialInfoPublisher = 514
    PUBSUB_TYPE_ERPCmdController = 515
    PUBSUB_TYPE_ERPObjectInfoPublisher = 516
    PUBSUB_TYPE_MORAI = 768
    PUBSUB_TYPE_MoraiAutowareInfoPublisher = 769
    PUBSUB_TYPE_MoraiInfoPublisher = 770
    PUBSUB_TYPE_MoraiCmdController = 771
    PUBSUB_TYPE_MoraiObjectInfoPublisher = 772
    PUBSUB_TYPE_MoraiSimProcHandleSubscriber = 773
    PUBSUB_TYPE_MoraiSimProcStatusPublisher = 774
    PUBSUB_TYPE_MoraiSLServiceProvider = 775
    PUBSUB_TYPE_MoraiLampsSubscriber = 776
    PUBSUB_TYPE_MoraiSimProcServiceProvider = 777
    PUBSUB_TYPE_MoraiTLServiceProvider = 778
    PUBSUB_TYPE_MoraiReplayInfoPublisher = 779
    PUBSUB_TYPE_MoraiEventCmdServiceProvider = 780
    PUBSUB_TYPE_MoraiVehicleSpecServiceProvider = 781
    PUBSUB_TYPE_MoraiMapSpecServiceProvider = 782
    PUBSUB_TYPE_MoraiSLSubscriber = 783
    PUBSUB_TYPE_AUTOWARE = 1024
    PUBSUB_TYPE_MoraiAutowareCmdController = 1025
    PUBSUB_TYPE_MoraiAutowareGear = 1026
    PUBSUB_TYPE_NAVER = 1280
    PUBSUB_TYPE_NaverSLSubscriber = 1281
    PUBSUB_TYPE_NaverSimProcHandleSubscriber = 1282
    PUBSUB_TYPE_NaverInfoPublisher_ObdAcceleratorPedal = 1283
    PUBSUB_TYPE_NaverInfoPublisher_ObdGear = 1284
    PUBSUB_TYPE_NaverInfoPublisher_ObdLamps = 1285
    PUBSUB_TYPE_NaverInfoPublisher_ObdSteeringAngle = 1286
    PUBSUB_TYPE_NaverInfoPublisher_ObdWheelSpeeds = 1287
    PUBSUB_TYPE_NaverInfoPublisher_ObdYawRate = 1288
    PUBSUB_TYPE_NaverInfoPublisher_VehiclePose = 1289
    PUBSUB_TYPE_NaverObjectInfoPublisher = 1290
    PUBSUB_TYPE_NaverSimProcStatusPublisher = 1291
    PUBSUB_TYPE_NaverTrafficLightPublisher = 1292
    PUBSUB_TYPE_NaverCmdController = 1293
    PUBSUB_TYPE_NaverSubscriber_ObdLamps = 1294
    PUBSUB_TYPE_NaverSLServiceProvider = 1295
    PUBSUB_TYPE_NaverSimProcServiceProvider = 1296
    PUBSUB_TYPE_NaverTLServiceProvider = 1297
    PUBSUB_TYPE_CBNU = 1536
    PUBSUB_TYPE_CBNUSLSubscriber = 1537
    PUBSUB_TYPE_CBNUCmdController = 1538
    PUBSUB_TYPE_CBNUInfoPublisher = 1539
    PUBSUB_TYPE_CBNUObjectInfoPublisher = 1540
    PUBSUB_TYPE_KAIST = 1792
    PUBSUB_TYPE_KaistSLSubscriber = 1793
    PUBSUB_TYPE_KaistCmdController = 1794
    PUBSUB_TYPE_KaistInfoPublisher = 1795
    PUBSUB_TYPE_KaistObjectInfoPublisher = 1796
    PUBSUB_TYPE_WECAR = 2048
    PUBSUB_TYPE_WeCarCmdController = 2049
    PUBSUB_TYPE_WeCarInfoPublisher = 2050
    PUBSUB_TYPE_WeCarInfoVescStatePublisher = 2051
    PUBSUB_TYPE_WeCarInfoSteerPublisher = 2052
    PUBSUB_TYPE_SSAFY = 2304
    PUBSUB_TYPE_SSAFYCmdController = 2305
    PUBSUB_TYPE_SSAFYTurtleBotInfoPublisher = 2306
    PUBSUB_TYPE_SSAFYObjectStatusPublisher = 2307
    PUBSUB_TYPE_SSAFYObjectControlSubscriber = 2308
    PUBSUB_TYPE_SSAFYEnviromentInfoPublisher = 2309
    PUBSUB_TYPE_SSAFYStuffObjectControlSubscriber = 2310
    PUBSUB_TYPE_SSAFYStuffObjectInfoPublisher = 2311
    PUBSUB_TYPE_KAISTGHOST = 2560
    PUBSUB_TYPE_KaistGhostCmdController = 2561
    PUBSUB_TYPE_MORAIGHOST = 2816
    PUBSUB_TYPE_MoraiGhostCmdController = 2817
    PUBSUB_TYPE_MoraiNpcGhostController = 2818
    PUBSUB_TYPE_SCOUTMINI = 3072
    PUBSUB_TYPE_ScoutCmdController = 3073
    PUBSUB_TYPE_ScoutInfoPublisher = 3074
    PUBSUB_TYPE_ScoutLightCmdSubscriber = 3075
    PUBSUB_TYPE_CYBER = 3328
    PUBSUB_TYPE_CyberCmdController = 3329
    PUBSUB_TYPE_CyberChassisPublisher = 3330
    PUBSUB_TYPE_CyberBestPosePublisher = 3331
    PUBSUB_TYPE_CyberInsStatPublisher = 3332
    PUBSUB_TYPE_CyberOdometryPublisher = 3333
    PUBSUB_TYPE_CyberCorrectedImuPublisher = 3334
    PUBSUB_TYPE_CyberImuPublisher = 3335
    PUBSUB_TYPE_ERPV2 = 3584
    PUBSUB_TYPE_ERPv2CmdController = 3585
    PUBSUB_TYPE_ERPv2InfoPublisher = 3586
    PUBSUB_TYPE_ERPv2ObjectInfoPublisher = 3587
    PUBSUB_TYPE_ERPv2SerialInfoPublisher = 3588
    PUBSUB_TYPE_WEBOT = 3840
    PUBSUB_TYPE_WeBOTCmdController = 3841
    PUBSUB_TYPE_WeBOTInfoSteerPublisher = 3842
    PUBSUB_TYPE_WeBOTInfoVescStatePublisher = 3843
    PUBSUB_TYPE_PR = 4096
    PUBSUB_TYPE_PRCmdController = 4097
    PUBSUB_TYPE_PRStatusPublisher = 4098
    PUBSUB_TYPE_PREventServiceProvider = 4099
    PUBSUB_TYPE_DIFFERETIAL_DRIVE = 4352
    PUBSUB_TYPE_MoraiDdCmdController = 4353
    PUBSUB_TYPE_MoraiDdInfoPublisher = 4354
    PUBSUB_TYPE_END = 4355

class NetworkConnectType(IntEnum):
    NETWORK_CONNECT_TYPE_BEGIN = 0
    NETWORK_CONNECT_TYPE_EGO = 1
    NETWORK_CONNECT_TYPE_PUBSUB = 2
    NETWORK_CONNECT_TYPE_SIM = 3
    NETWORK_CONNECT_TYPE_END = 4

class TrafficLightColor(IntEnum) :
    TL_COLOR_BEGIN = 0
    TL_COLOR_R = 1				    # Red
    TL_COLOR_Y = 4				    # Yellow
    TL_COLOR_SG = 16				# Straigt Green >> 직진 녹색
    TL_COLOR_LG = 32				# Left    Green >> ← 좌회전 녹색
    TL_COLOR_RG = 64				# Right   Green >> → 우회전 녹색
    TL_COLOR_UTG = 128				# UTurn   Green 
    TL_COLOR_ULG = 256				# Upper Left  Green >> ↖
    TL_COLOR_URG = 512				# Upper Right Green >> ↗
    TL_COLOR_LLG = 1024				# Lower Left  Green >> ↙
    TL_COLOR_LRG = 2048				# Lower Right Green >> ↘ 
    
    TL_COLOR_R_WITH_Y = 5		    # R | Y
    TL_COLOR_Y_WITH_G = 20		    # Y | SG
    TL_COLOR_Y_WITH_GLEFT = 36		# Y | LG
    TL_COLOR_G_WITH_GLEFT = 48		# SG | LG
    TL_COLOR_R_WITH_GLEFT = 33		# R | LG
    TL_COLOR_LLG_SG = 1040		    # SG | LLG
    TL_COLOR_R_LLG = 1025		    # R | LLG
    TL_COLOR_ULG_URG = 768		    # ULG | URG

    TL_COLOR_NONE = 0               # 불이 꺼져있는 상태
    TL_COLOR_UNDEFINED = -1         # 신호등이 초기화 되지 않았을 때
    TL_COLOR_NOT_DETECTED = -2

class CruiseType(IntEnum):
    CRUISE_TYPE_BEGIN = 0
    CRUISE_TYPE_LINK = 1
    CRUISE_TYPE_CONSTANT = 2
    CRUISE_TYPE_END = 3


class VehicleType(IntEnum):
    VEHICLE_TYPE_UNKNOWN = 0
    VEHICLE_TYPE_MASTER = 1
    VEHICLE_TYPE_MULTI_EGO = 2
    VEHICLE_TYPE_SURROUND = 3

class ActorInfo:
    def __init__(self, role_name = '-1', actor_type = ActorType.ACTOR_TYPE_VEHICLE):
        self.role_name = role_name
        self.actor_type = actor_type

class BoundingBox:
    def __init__(self, extent=0, location=0, rotation=0):
        self.extent = extent
        self.location = location
        self.rotation = rotation


class LinkInfo:
    def __init__(self):
        self.unique_id = ''
        self.waypoint_idx = 0

class VehicleAuxState:
    def __init__(self):
        self.current_link_info = LinkInfo()
        self.remaining_distance = 0.0
        self.is_pass_dest_pos = False
        self.tl_id = ''
        self.tl_color = TrafficLightColor.TL_COLOR_BEGIN
        self.is_collision = False

class VehicleState:
    def __init__(self):
        self.throttle = 0.0
        self.steer = 0.0
        self.brake = 0.0
        self.front_wheel_angle = 0.0
        self.gear_mode = GearMode.GEAR_MODE_D
        self.vehicle_aux_state = VehicleAuxState()

class ActorState:
    def __init__(self):
        self.actor_info = ActorInfo()
        self.bounding_box = BoundingBox()
        self.transform = Transform()
        self.velocity = Vector3()
        self.angular_velocity = Vector3()
        self.acceleration = Vector3()
        
        self.vehicle_state = VehicleState()

    def get_current_lane_id(self):
        return self.vehicle_state.vehicle_aux_state.current_link_info.unique_id

class Vector3:
    def __init__(self, x = 0, y = 0, z = 0):
        self.x = x
        self.y = y
        self.z = z


class Location(Vector3):
    def __init__(self, x = 0, y = 0, z = 0):
        self.x = x
        self.y = y
        self.z = z

    def distance(self, location):
        dest = np.array([location.x, location.y, location.z])
        src = np.array([self.x, self.y, self.z])
        vect = dest - src
        distance = np.linalg.norm(vect, ord=2)
        return distance

    def is_empty(self):
        return True if (self.x == 0 and self.y == 0 and self.z == 0) else False
        
    def get_dir_vector(self, location) :
        dest = np.array([location.x, location.y, location.z])
        src = np.array([self.x, self.y, self.z])
        dir_vec = dest - src
        norm_dir_vec = dir_vec / np.linalg.norm(dir_vec, ord=2)
        return norm_dir_vec


class Rotation:
    def __init__(self, roll = 0, pitch = 0, yaw = 0):
        self.roll = roll
        self.pitch = pitch
        self.yaw = yaw

    def get_forward_vector(self):        
        vector = Vector3()        
        return vector

    def is_empty(self):
        return True if (self.roll == 0 and self.pitch == 0 and self.yaw == 0) else False


class Transform:
    def __init__(self, location=Location(), rotation=Rotation()):
        self.location = location
        self.rotation = rotation

    def is_empty(self):
        return True if (self.location.is_empty() and self.rotation.is_empty()) else False

    def calculate_rotation(self, target_transform : 'Transform') :
        '''해당 waypoint 를 바라보는 yaw값을 설정'''
        # norm_dir_vec = self.location.get_dir_vector(target_transform.location)
        target_pos = np.array([target_transform.location.x, target_transform.location.y, target_transform.location.z])
        cur_pos = np.array([self.location.x, self.location.y, self.location.z])

        dir_vec = target_pos - cur_pos
        norm_dir_vector = np.array([dir_vec[0], dir_vec[1], 0])
        norm_dir_vector = norm_dir_vector / np.linalg.norm(norm_dir_vector, ord=2)
        yaw_actan = np.math.atan2(norm_dir_vector[1], norm_dir_vector[0])
        yaw_deg = yaw_actan * (180 / np.pi)

        self.rotation.roll = 0
        self.rotation.pitch = 0
        self.rotation.yaw = yaw_deg

        return self

    def transform(self, transform : 'Transform'):
        world_tr = Transform()
        world2self_tr = get_tr(self) # 월드 좌표축 -> self 좌표축, self 좌표 -> 월드 좌표
        self2target_tr = get_tr(transform)
        world2target_tr = world2self_tr @ self2target_tr
        
        r = world2target_tr[:3, :3]
        rpy = r2rpy(r)
        world_tr.location.x, world_tr.location.y, world_tr.location.z = world2target_tr[:3, 3]
        world_tr.rotation.roll = rpy[0]
        world_tr.rotation.pitch = rpy[1]
        world_tr.rotation.yaw = rpy[2]

        return world_tr

    def get_transform_to(self, distance, ObjectParams : 'ObjectParams'):
        
        _g90_front = 4.02
        _model = ObjectParams.object_name
        _scale = ObjectParams.scale

        if _model == "OBJ_Hyundai_Grandeur":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*1.12, y=0, z=_scale.z*0.34), Rotation(yaw=0))
        elif _model == "OBJ_Hyundai_Universe_High":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*3.24, y=0, z=_scale.z*0.53), Rotation(yaw=0))
        elif _model == "OBJ_Hyundai_Sonata":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*1.085, y=0, z=_scale.z*0.33), Rotation(yaw=0))
        elif _model == "SportBike":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*0.815), Rotation(yaw=0))
        elif _model == "CargoBox":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*0.375), Rotation(yaw=0))
        elif _model == "OBJ_Boy":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*0.1), Rotation(yaw=0))
        elif _model == "OBJ_Man":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*0.15), Rotation(yaw=0))
        elif _model == "Bike1":
            transform = Transform(Location(x=_g90_front+distance+_scale.x*0.825), Rotation(yaw=180))
        elif _model == "ChessboardObject":
            transform = Transform(Location(x=_g90_front+distance, y=0, z=1+_scale.z*0.75), Rotation(yaw=180))
        else:
            raise ValueError(f"{_model} does not exist.")
        
        return transform

class ObjectParams:
    def __init__(self):
        self.scale = None
        self.object_name = 'WoodBox'


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
import morai_sim_info_pb2 as morai__sim__info__pb2
import morai_type_pb2 as morai__type__pb2


class MoraiSimInfoStub(object):
    """MORAI SIM 정보 확인을 위한 서비스
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetSimulatorVersion = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetSimulatorVersion',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=morai__type__pb2.StringValue.FromString,
                )
        self.GetTimestamp = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetTimestamp',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=morai__type__pb2.Int64Value.FromString,
                )
        self.GetAvailableMaps = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetAvailableMaps',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=morai__type__pb2.StringList.FromString,
                )
        self.GetActorTypes = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetActorTypes',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=morai__type__pb2.StringList.FromString,
                )
        self.GetAvailableObject = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetAvailableObject',
                request_serializer=morai__sim__info__pb2.CategoryObstacles.SerializeToString,
                response_deserializer=morai__sim__info__pb2.AvailableObject.FromString,
                )
        self.GetDataPath = channel.unary_unary(
                '/morai_sim_api.MoraiSimInfo/GetDataPath',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=morai__type__pb2.StringValue.FromString,
                )


class MoraiSimInfoServicer(object):
    """MORAI SIM 정보 확인을 위한 서비스
    """

    def GetSimulatorVersion(self, request, context):
        """시뮬레이터 버전 확인
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTimestamp(self, request, context):
        """시뮬레이터의 Timestamp 확인
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAvailableMaps(self, request, context):
        """현재 사용자가 사용 가능한 맵 리스트 조회
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetActorTypes(self, request, context):
        """시뮬레이터에서 사용 가능한 Actor 종류 조회
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAvailableObject(self, request, context):
        """Get Available Object matched to the user account info
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDataPath(self, request, context):
        """데이터 저장 경로 획득
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MoraiSimInfoServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetSimulatorVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSimulatorVersion,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=morai__type__pb2.StringValue.SerializeToString,
            ),
            'GetTimestamp': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTimestamp,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=morai__type__pb2.Int64Value.SerializeToString,
            ),
            'GetAvailableMaps': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAvailableMaps,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=morai__type__pb2.StringList.SerializeToString,
            ),
            'GetActorTypes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetActorTypes,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=morai__type__pb2.StringList.SerializeToString,
            ),
            'GetAvailableObject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAvailableObject,
                    request_deserializer=morai__sim__info__pb2.CategoryObstacles.FromString,
                    response_serializer=morai__sim__info__pb2.AvailableObject.SerializeToString,
            ),
            'GetDataPath': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDataPath,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=morai__type__pb2.StringValue.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'morai_sim_api.MoraiSimInfo', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MoraiSimInfo(object):
    """MORAI SIM 정보 확인을 위한 서비스
    """

    @staticmethod
    def GetSimulatorVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetSimulatorVersion',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            morai__type__pb2.StringValue.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTimestamp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetTimestamp',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            morai__type__pb2.Int64Value.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAvailableMaps(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetAvailableMaps',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            morai__type__pb2.StringList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetActorTypes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetActorTypes',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            morai__type__pb2.StringList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAvailableObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetAvailableObject',
            morai__sim__info__pb2.CategoryObstacles.SerializeToString,
            morai__sim__info__pb2.AvailableObject.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDataPath(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/morai_sim_api.MoraiSimInfo/GetDataPath',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            morai__type__pb2.StringValue.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requirements = ['grpcio==1.44.0', 'grpcio-tools==1.44.0', 'numpy', 'spatialmath-python==1.0.3']

setuptools.setup(
    name="morai-kit",
    version="1.1.0",
    author="ischoi",
    author_email="ischoi@morai.ai",
    description="MORAI Sensor Kit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=requirements,
    include_package_data=True,
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: morai_type.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10morai_type.proto\x12\rmorai_sim_api\".\n\x06Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"(\n\x05Point\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\"*\n\x07Vector3\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\"_\n\tTransform\x12(\n\x08location\x18\x01 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12(\n\x08rotation\x18\x02 \x01(\x0b\x32\x16.morai_sim_api.Vector3\"\x89\x01\n\x0b\x42oundingBox\x12&\n\x06\x65xtent\x18\x01 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12(\n\x08location\x18\x02 \x01(\x0b\x32\x16.morai_sim_api.Vector3\x12(\n\x08rotation\x18\x03 \x01(\x0b\x32\x16.morai_sim_api.Vector3\"\x1c\n\x0bStringValue\x12\r\n\x05value\x18\x01 \x01(\t\"\x1c\n\nStringList\x12\x0e\n\x06values\x18\x01 \x03(\t\"\x1b\n\nInt32Value\x12\r\n\x05value\x18\x01 \x01(\x05\"\x1b\n\nInt64Value\x12\r\n\x05value\x18\x01 \x01(\x03\"\x1c\n\x0bUint32Value\x12\r\n\x05value\x18\x01 \x01(\r\"\x1c\n\x0bUint64Value\x12\r\n\x05value\x18\x01 \x01(\x04\"\x1b\n\nFloatValue\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1c\n\x0b\x44oubleValue\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1a\n\tBoolValue\x12\r\n\x05value\x18\x01 \x01(\x08\"3\n\x08LinkInfo\x12\x11\n\tunique_id\x18\x01 \x01(\t\x12\x14\n\x0cwaypoint_idx\x18\x02 \x01(\x05\x42&H\x01\xaa\x02!CoreLib.Grpc.Protobuf.MoraiSimAPIb\x06proto3')



_RESULT = DESCRIPTOR.message_types_by_name['Result']
_POINT = DESCRIPTOR.message_types_by_name['Point']
_VECTOR3 = DESCRIPTOR.message_types_by_name['Vector3']
_TRANSFORM = DESCRIPTOR.message_types_by_name['Transform']
_BOUNDINGBOX = DESCRIPTOR.message_types_by_name['BoundingBox']
_STRINGVALUE = DESCRIPTOR.message_types_by_name['StringValue']
_STRINGLIST = DESCRIPTOR.message_types_by_name['StringList']
_INT32VALUE = DESCRIPTOR.message_types_by_name['Int32Value']
_INT64VALUE = DESCRIPTOR.message_types_by_name['Int64Value']
_UINT32VALUE = DESCRIPTOR.message_types_by_name['Uint32Value']
_UINT64VALUE = DESCRIPTOR.message_types_by_name['Uint64Value']
_FLOATVALUE = DESCRIPTOR.message_types_by_name['FloatValue']
_DOUBLEVALUE = DESCRIPTOR.message_types_by_name['DoubleValue']
_BOOLVALUE = DESCRIPTOR.message_types_by_name['BoolValue']
_LINKINFO = DESCRIPTOR.message_types_by_name['LinkInfo']
Result = _reflection.GeneratedProtocolMessageType('Result', (_message.Message,), {
  'DESCRIPTOR' : _RESULT,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Result)
  })
_sym_db.RegisterMessage(Result)

Point = _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
  'DESCRIPTOR' : _POINT,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Point)
  })
_sym_db.RegisterMessage(Point)

Vector3 = _reflection.GeneratedProtocolMessageType('Vector3', (_message.Message,), {
  'DESCRIPTOR' : _VECTOR3,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Vector3)
  })
_sym_db.RegisterMessage(Vector3)

Transform = _reflection.GeneratedProtocolMessageType('Transform', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORM,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Transform)
  })
_sym_db.RegisterMessage(Transform)

BoundingBox = _reflection.GeneratedProtocolMessageType('BoundingBox', (_message.Message,), {
  'DESCRIPTOR' : _BOUNDINGBOX,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.BoundingBox)
  })
_sym_db.RegisterMessage(BoundingBox)

StringValue = _reflection.GeneratedProtocolMessageType('StringValue', (_message.Message,), {
  'DESCRIPTOR' : _STRINGVALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.StringValue)
  })
_sym_db.RegisterMessage(StringValue)

StringList = _reflection.GeneratedProtocolMessageType('StringList', (_message.Message,), {
  'DESCRIPTOR' : _STRINGLIST,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.StringList)
  })
_sym_db.RegisterMessage(StringList)

Int32Value = _reflection.GeneratedProtocolMessageType('Int32Value', (_message.Message,), {
  'DESCRIPTOR' : _INT32VALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Int32Value)
  })
_sym_db.RegisterMessage(Int32Value)

Int64Value = _reflection.GeneratedProtocolMessageType('Int64Value', (_message.Message,), {
  'DESCRIPTOR' : _INT64VALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Int64Value)
  })
_sym_db.RegisterMessage(Int64Value)

Uint32Value = _reflection.GeneratedProtocolMessageType('Uint32Value', (_message.Message,), {
  'DESCRIPTOR' : _UINT32VALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Uint32Value)
  })
_sym_db.RegisterMessage(Uint32Value)

Uint64Value = _reflection.GeneratedProtocolMessageType('Uint64Value', (_message.Message,), {
  'DESCRIPTOR' : _UINT64VALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.Uint64Value)
  })
_sym_db.RegisterMessage(Uint64Value)

FloatValue = _reflection.GeneratedProtocolMessageType('FloatValue', (_message.Message,), {
  'DESCRIPTOR' : _FLOATVALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.FloatValue)
  })
_sym_db.RegisterMessage(FloatValue)

DoubleValue = _reflection.GeneratedProtocolMessageType('DoubleValue', (_message.Message,), {
  'DESCRIPTOR' : _DOUBLEVALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.DoubleValue)
  })
_sym_db.RegisterMessage(DoubleValue)

BoolValue = _reflection.GeneratedProtocolMessageType('BoolValue', (_message.Message,), {
  'DESCRIPTOR' : _BOOLVALUE,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.BoolValue)
  })
_sym_db.RegisterMessage(BoolValue)

LinkInfo = _reflection.GeneratedProtocolMessageType('LinkInfo', (_message.Message,), {
  'DESCRIPTOR' : _LINKINFO,
  '__module__' : 'morai_type_pb2'
  # @@protoc_insertion_point(class_scope:morai_sim_api.LinkInfo)
  })
_sym_db.RegisterMessage(LinkInfo)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\001\252\002!CoreLib.Grpc.Protobuf.MoraiSimAPI'
  _RESULT._serialized_start=35
  _RESULT._serialized_end=81
  _POINT._serialized_start=83
  _POINT._serialized_end=123
  _VECTOR3._serialized_start=125
  _VECTOR3._serialized_end=167
  _TRANSFORM._serialized_start=169
  _TRANSFORM._serialized_end=264
  _BOUNDINGBOX._serialized_start=267
  _BOUNDINGBOX._serialized_end=404
  _STRINGVALUE._serialized_start=406
  _STRINGVALUE._serialized_end=434
  _STRINGLIST._serialized_start=436
  _STRINGLIST._serialized_end=464
  _INT32VALUE._serialized_start=466
  _INT32VALUE._serialized_end=493
  _INT64VALUE._serialized_start=495
  _INT64VALUE._serialized_end=522
  _UINT32VALUE._serialized_start=524
  _UINT32VALUE._serialized_end=552
  _UINT64VALUE._serialized_start=554
  _UINT64VALUE._serialized_end=582
  _FLOATVALUE._serialized_start=584
  _FLOATVALUE._serialized_end=611
  _DOUBLEVALUE._serialized_start=613
  _DOUBLEVALUE._serialized_end=641
  _BOOLVALUE._serialized_start=643
  _BOOLVALUE._serialized_end=669
  _LINKINFO._serialized_start=671
  _LINKINFO._serialized_end=722
# @@protoc_insertion_point(module_scope)
